# -*- coding: UTF-8 -*-
##############################################################################
# Copyright (c) 2011 Timothy W. Cook and Contributors.
# All Rights Reserved. See CONTRIBUTORS.txt
#
# This software is subject to the provisions of the GNU LESSER GENERAL 
# PUBLIC LICENSE Version 3, Dated: 29 June 2007 See docs/LICENSE.txt or 
# http://www.gnu.org/licenses/lgpl.html 
##############################################################################

from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty

import interfaces 

class DvAny(object):

    implements(IDvAny)
    
    def __init__(self, vtb=None,vte=None,ev=None):
        self.vtb = vtb
        self.vte = vte
        self.ev = ev
        
class DvBoolean(DvAny):

    implements(IDvBoolean)

    def __init__(self, value, **kwargs):
        DvAny.__init__(self, **kwargs)
        
        self.dv = value

class DvText(DvAny):

    implements(IDvText)

    def __init__(self, dv='', lang='', **kwargs):
        DvAny.__init__(self, **kwargs) 
                       
        self.dv = dv
        self.language = lang
        
        
class DvParagraph(DvAny):

    implements(IDvParagraph)

    def __init__(self,items, **kwargs):
        DvAny.__init__(self, **kwargs) 
                       
        self.items = items


class DvCodedText(DvText):
    
    implements(IDvCodedText)

    def __init__(self, cs='', tn='', ta='', dv='', lang='', **kwargs):
        DvText.__init__(self,dv,lang, **kwargs) 
        self.code_string = cs
        self.terminology_name = tn
        self.terminology_abbrev = ta
                       
 
class DvIdentifier(DvText):

    grok.implements(IDvIdentifier)

    def __init__(self, issuer='', assigner='', name='', dv='', lang='', **kwargs):
        DvText.__init__(self,dv,lang, **kwargs) 
        
        self.issuer = issuer
        self.assigner = assigner
        self.name = name


class DvURI(DvAny):
    implements(IDvUri)

    def __init__(self, dv='', **kwargs):
        DvAny.__init__(self, **kwargs)
        
        self.dv = dv        

    def scheme(self):
        colon = self.dv.find(':')
        return self.dv[:colon]


    def path(self):
        colon = self.dv.find(':')
        return self.dv[colon+1:]

    def fragmentId(self,fragstr):
        if frastr in self.dv:
            return (self.dv.find(fragstr),fragstr)
        else:
            return (None,u"Fragment not found in URI." + self.dv)

    def query(self, qstr):
        if qstr in self.dv:
            return (self.dv.find(qstr),qstr)
        else:
            return (None,u"Query not found in URI." + self.dv)
