# -*- coding: UTF-8 -*-
##############################################################################
# Copyright (c) 2011 Timothy W. Cook and Contributors.
# All Rights Reserved. See CONTRIBUTORS.txt
#
# This software is subject to the provisions of the GNU LESSER GENERAL 
# PUBLIC LICENSE Version 3, Dated: 29 June 2007 See docs/LICENSE.txt or 
# http://www.gnu.org/licenses/lgpl.html 
##############################################################################

from zope.interface import Interface, Attribute
from zope.schema import Bool, TextLine, Object
from zope.i18nmessageid.message import MessageFactory

_ = MessageFactory('mlhim.rm')


class IAddress(Interface):
    """
    Address of contact.
    """

    details=Object(
        schema=IItemStructure,
        title=_(u"Details"),
        description=_(u"The details of the address."),
    )

    def type():
        """
        Return the type of address from 'name'.
        """

    def asString():
        """
        Address in the form of a string.
        """



class IPartyIdentity(Interface):
    """
    An identity owned by a party.
    """

    details=Object(
        schema=IItemStructure,
        title=_(u"Details"),
        description=_(u"The value of the identitiy"),
    )

    def asString():
        """
        Identity in the form of a string.
        """

    def purpose():
        """
        Purpose of Identity.
        Taken from value of inherited name attribute.
        """


class IContact(Interface):
    """
    Description of a means of contacting a party.
    """

    timeValidity=Object(
        schema=IDvInterval,
        title=_(u"Time Validity"),
        description=_(u"Valid time interval for this contact descriptor."),
        required=False,
    )

    addresses=List(
        value_type = Object(schema = IAddress),
        title=_(u"Addresses"),
        description=_(u"A set of addresses for this purpose and time frame."),
    )

    def purpose():
        """
        Purpose for which this contact is used.
        Taken from the inherited 'name' attribute.
        """
        return self.name


class IPartyRelationship(Interface):
    """
    Generic description of a relationship between parties.
    """

    uid=Object(
        schema=IUidBasedId,
        title=_(u"UID"),
        description=_(u"Identifier of this party."),

    )

    details=Object(
        schema=IItemStructure,
        title=_(u"Details"),
        description=(u"Description of the relationship."),
        required=False,
    )

    timeValidity=Object(
        schema=IDvInterval,
        title=_(u"Time Validity"),
        description=_(u"Valid time interval for this relationship."),
        required=False,
    )

    source=Object(
        schema=IPartyRef,
        title=_(u"Source"),
        description=_(u"Source of relationship."),

    )

    target=Object(
        schema=IPartyRef,
        title=_(u"Target"),
        description=_(u"Target of relationship."),

    )

    def type():
        """
        Type of relationship such as employment, authority, etc.
        """


class IParty(Interface):
    """
    Ancestor of all party types.
    """

    uid=Object(
        schema=IUidBasedId,
        title=_(u"UID"),
        description=_(u"Identifier of this party."),
    )

    identities=Set(
        title=_(u"Indentities"),
        description=_(u"Identities used by the party to identify itself."),
        value_type = Object(schema = IPartyIdentity))

    contacts=Set(
        title=_(u"Contacts"),
        description=_(u"Contacts for this party."),
        value_type = Object(schema = IContact))

    relationships=Set(
        title=_(u"Relationships"),
        description=_(u"Relationships in which this role takes part as"
                      " source."),
        required=False,
        value_type = Object(schema = IPartyRelationship))

    reverseRelationships=Set(
        title=_(u"Reverse Relationships"),
        description=_(u"Relationships in which this role takes part as source."),
        required=False,
        value_type = Object(schema = ILocatableRef))

    details=Object(
        schema=IItemStructure,
        title=_(u"Details"),
        description=_(u"All other party details."),
    )

    def type():
        """
        Return the type of party from the inherited 'name' attribute.
        """



class IActor(Interface):
    """
    Ancestor of all real world types.
    """

    roles=Set(
        title=_(u"Roles"),
        description=_(u"Identifiers of the Version container for "
                      "each Role played by this party."),
        required=False,
        value_type = Object(schema = IPartyRef))

    languages=List(
        title=_(u"Languages"),
        description=_(u"A list of languages to be used to communice"
                      " with this actor."),
        required=False,
        value_type = Object(schema = IDvText))

    def hasLegalIdentity():
        """
        Return True if there is an identity with purpose 'legal identity'
        """



class IAgent(IActor):
    """
    Generic concept of of any kind of agent including devices.
    """
    pass

class ICapability(Interface):
    """
    Capability of a role such as ehr modifier, healthcare provider, etc.
    """

    credentials=Object(
        schema=IItemStructure,
        title=_(u"Credentials"),
        description=_(u"Qualifications of the performer of the role."),

    )

    timeValidity=Object(
        schema=IDvInterval,
        title=_(u"Time Validity"),
        description=_(u"Valid time interval for the credentials of this capability."),
        required=False,
    )
class ICapability(Interface):
    """
    Capability of a role such as ehr modifier, healthcare provider, etc.
    """

    credentials=Object(
        schema=IItemStructure,
        title=_(u"Credentials"),
        description=_(u"Qualifications of the performer of the role."),

    )

    timeValidity=Object(
        schema=IDvInterval,
        title=_(u"Time Validity"),
        description=_(u"Valid time interval for the credentials of this capability."),
        required=False,
    )
class ICapability(Interface):
    """
    Capability of a role such as ehr modifier, healthcare provider, etc.
    """

    credentials=Object(
        schema=IItemStructure,
        title=_(u"Credentials"),
        description=_(u"Qualifications of the performer of the role."),

    )

    timeValidity=Object(
        schema=IDvInterval,
        title=_(u"Time Validity"),
        description=_(u"Valid time interval for the credentials of this capability."),
        required=False,
    )
class ICapability(Interface):
    """
    Capability of a role such as ehr modifier, healthcare provider, etc.
    """

    credentials=Object(
        schema=IItemStructure,
        title=_(u"Credentials"),
        description=_(u"Qualifications of the performer of the role."),

    )

    timeValidity=Object(
        schema=IDvInterval,
        title=_(u"Time Validity"),
        description=_(u"Valid time interval for the credentials of this capability."),
        required=False,
    )



class IGroup(IActor):
    """
    A real world group of parties.
    """
    pass



class IOrganisation(IActor):
    """
    Generic descriptions of organizations.
    """
    pass



class IPerson(IActor):
    """
    Generic description of of persons.  Provides a dedicated type to whicih Person archetypes can be targeted."),
    """
    pass



class IRole(Interface):
    """
    Generic role played by a party.
    """

    capabilities=List(
        title=_(u"Capabilities"),
        description=_(u"Capabilities of this role."),
        required=False,
        value_type = Object(schema = ICapability)
    )

    timeValidity=Object(
        schema=IDvInterval,
        title=_(u"Time Validity"),
        description=_(u"Valid time interval for this role."),
        required=False,
    )

    performer=Object(
        schema=IPartyRef,
        title=_(u"Performer"),
        description=_(u"Reference to Version container of Actor playing this role."),
    )



class ISearchableActor(Interface):

    relationships = Attribute("")

    reverse_relationships = Attribute("")
    legal_identity = Attribute("")
